
import 'package:bar_tender_api_dart/src/models/Model.dart';
import 'package:http/http.dart' as http;

abstract class ApiClient<E> {
  final Duration timeout = const Duration(seconds: 5);
  final headers = {'Content-Type': 'application/json'};

  Future<E> fetch(Model model);

  Future<http.Response> createOrUpdate(Model model, dynamic object);

  Future<http.Response> genericGet(String requestPath) {
    final url = Uri.parse(requestPath);
    return http.get(
      url,
      headers: headers,
    );
  }

  Future<http.Response> genericPost(String requestPath, String objectToPost) {
    final url = Uri.parse(requestPath);
    return http.post(
      url,
      headers: headers,
      body: objectToPost,
    );
  }

  Future<http.Response> genericDelete(String requestPath, String? objectToDelete) {
    final url = Uri.parse(requestPath);
    if (objectToDelete == null) {
      return http.delete(url);
    }
    return http.delete(
      url,
      headers: headers,
      body: objectToDelete,
    );
  }


}