import 'dart:convert';

import 'package:bar_tender_api_dart/src/apiClients/EntityApiClient.dart';
import 'package:bar_tender_api_dart/src/models/Model.dart';
import 'package:bar_tender_api_dart/src/chart_item/ChartItem.dart';
import 'package:bar_tender_api_dart/src/paths.dart';
import 'package:http/http.dart';

import 'DrinkOrderUnified.dart';

class ChartItemUnifiedModel extends Model<DrinkOrderUnified> {

  int? idCustomer;

  ChartItemUnifiedModel(this.idCustomer,)
      : super(apiClient: EntityApiClient<DrinkOrderUnified>(), value: DrinkOrderUnified.empty(), urlPath: _buildUrl(idCustomer));

  static String _getUrlChart(int? isCustomer) {
    final customerIdParam = isCustomer != null ? "idUser=$isCustomer" : "";
    return "$UPDATE_CHART_QUEUE?$customerIdParam";
  }

  static String _getUrlDeleteChart(int? idChartItem) {
    return "$DELETE_CHART_QUEUE$idChartItem";
  }

  static String _buildUrl(int? idCustomer) {
    final customerIdParam = idCustomer != null ? "idCustomer=$idCustomer" : "";
    return "$UNIFIED_ORDER_BY_ID?$customerIdParam";
  }

  void updateUrl(int? idCustomer) {
    this.idCustomer = idCustomer;
    urlPath = _buildUrl(this.idCustomer);
  }

  Future<bool> updateChartList(List<ChartItem> chartItems) async {
    if (idCustomer == null) return false;
    print(jsonEncode(chartItems));
    var response = await apiClient.genericPost(
      _getUrlChart(idCustomer),
      jsonEncode(chartItems),
    );
    return response.statusCode == 200;
  }

  Future<bool> deleteChartItem(ChartItem chartItem) async {
    if (idCustomer == null || chartItem.id == null || idCustomer != chartItem.idCustomer) return false;
    var response = await apiClient.genericDelete(
      _getUrlDeleteChart(chartItem.id),
      null,
    );
    return response.statusCode == 200;
  }

  @override
  String toJson(DrinkOrderUnified element) => jsonEncode(element);

  @override
  DrinkOrderUnified fromJson(dynamic json) => DrinkOrderUnified.fromJson(json);

  @override
  List<ChartItem> toList() {
    return value.chartItem;
  }

  @override
  bool isValueEmpty() {
    return value.customer.id == 0;
  }

  Future<Response> setDispenserElement(ChartItem first, int idDispenser) async {
    ChartItem toUpdate = ChartItem.copy(first);
    toUpdate.idDispenser = idDispenser;
    return apiClient.genericPost(CHART_ITEM_API_PATH, json.encode(toUpdate));
  }

}