
import 'package:bar_tender_api_dart/src/drinkUnified/DrinkUnifiedRecipe.dart';
import 'package:bar_tender_api_dart/src/ingredient/Ingredient.dart';
import 'package:bar_tender_api_dart/src/recipe_item/RecipeItem.dart';
import 'package:json_annotation/json_annotation.dart';
part 'DrinkUnified.g.dart';

@JsonSerializable()
class DrinkUnified {
  final Map<int, DrinkUnifiedRecipe> unifiedDrinks;
  final Map<int, Ingredient> ingredients;

  DrinkUnified({
    required this.unifiedDrinks,
    required this.ingredients,
  });

  factory DrinkUnified.fromJson(Map<String, dynamic> json) => _$DrinkUnifiedFromJson(json);
  Map<String, dynamic> toJson() => _$DrinkUnifiedToJson(this);

  factory DrinkUnified.empty() {
    return DrinkUnified(
      unifiedDrinks: {},
      ingredients: {}
    );
  }

  Iterable<DrinkUnifiedRecipe> unifiedDrinkList () {
    return unifiedDrinks.values;
  }

  static String percentualeAlcolica(DrinkUnified drinkUnified, List<RecipeItem> listRecipeItem) {
    double totalAlcoholPercentage = 0.0;
    double totalVolume = 0.0;

    for (var recipeItem in listRecipeItem) {
      var ingredient = drinkUnified.ingredients[recipeItem.idIngredient];
      if (ingredient == null) return "Errore";
      double percentageOfIngredient = recipeItem.quantityPercent;
      double alcoholPercentageOfIngredient = ingredient.alcoholicPercentage;

      double alcoholVolume = (percentageOfIngredient / 100) * alcoholPercentageOfIngredient;
      totalAlcoholPercentage += alcoholVolume;
      totalVolume += percentageOfIngredient;
    }

    return ((totalAlcoholPercentage / totalVolume) * 100).toStringAsFixed(1);
  }

}