
import 'package:bar_tender_api_dart/src/drink/Drink.dart';
import 'package:bar_tender_api_dart/src/recipe_item/RecipeItem.dart';
import 'package:bar_tender_api_dart/src/recipe_item_bar_custom/RecipeItemBarCustom.dart';
import 'package:bar_tender_api_dart/src/recipe_item_custom/RecipeItemCustom.dart';
import 'package:json_annotation/json_annotation.dart';
part 'DrinkUnifiedRecipe.g.dart';

@JsonSerializable()
class DrinkUnifiedRecipe {
  final Drink drink;
  final Map<String, RecipeItem>? recipeItems;
  final Map<String, RecipeItemCustom>? recipeItemsCustom;
  final Map<String, RecipeItemBarCustom>? recipeItemsBarCustom;

  DrinkUnifiedRecipe({
    required this.drink,
    required this.recipeItems,
    required this.recipeItemsCustom,
    required this.recipeItemsBarCustom,
  });

  factory DrinkUnifiedRecipe.fromJson(Map<String, dynamic> json) => _$DrinkUnifiedRecipeFromJson(json);
  Map<String, dynamic> toJson() => _$DrinkUnifiedRecipeToJson(this);

  factory DrinkUnifiedRecipe.empty() {
    return DrinkUnifiedRecipe(
      drink: Drink.empty(),
      recipeItems: {},
      recipeItemsCustom: {},
      recipeItemsBarCustom: {},
    );
  }

  factory DrinkUnifiedRecipe.copy(DrinkUnifiedRecipe drinkUnifiedRecipe) {
    // Copia il Drink
    Drink copiedDrink = Drink.copy(drinkUnifiedRecipe.drink);

    // Copia la mappa recipeItems
    Map<String, RecipeItem>? copiedRecipeItems;
    if (drinkUnifiedRecipe.recipeItems != null) {
      copiedRecipeItems = {};
      drinkUnifiedRecipe.recipeItems!.forEach((key, value) {
        copiedRecipeItems![key] = RecipeItem.copy(value);
      });
    }

    // Copia la mappa recipeItemsCustom
    Map<String, RecipeItemCustom>? copiedRecipeItemsCustom;
    if (drinkUnifiedRecipe.recipeItemsCustom != null) {
      copiedRecipeItemsCustom = {};
      drinkUnifiedRecipe.recipeItemsCustom!.forEach((key, value) {
        copiedRecipeItemsCustom![key] = RecipeItemCustom.copy(value);
      });
    }

    // Copia la mappa recipeItemsBarCustom
    Map<String, RecipeItemBarCustom>? copiedRecipeItemsBarCustom;
    if (drinkUnifiedRecipe.recipeItemsBarCustom != null) {
      copiedRecipeItemsBarCustom = {};
      drinkUnifiedRecipe.recipeItemsBarCustom!.forEach((key, value) {
        copiedRecipeItemsBarCustom![key] = RecipeItemBarCustom.copy(value);
      });
    }

    return DrinkUnifiedRecipe(
      drink: copiedDrink,
      recipeItems: copiedRecipeItems,
      recipeItemsCustom: copiedRecipeItemsCustom,
      recipeItemsBarCustom: copiedRecipeItemsBarCustom,
    );
  }

}