
import 'dart:convert';

import 'package:bar_tender_api_dart/src/apiClients/ApiClient.dart';
import 'package:flutter/material.dart';

abstract class Model<E> extends ChangeNotifier {
  final ApiClient<E> apiClient;
  String urlPath;
  E _value;
  String errorMessage = "";
  bool alreadyFetch = false; // return true only if correctly fetch
  List toList();
  bool isValueEmpty();

  Model({
    required this.urlPath,
    required this.apiClient,
    required E value
  }) : _value = value;

  E get value => _value;
  set setValue(E newValue) {
    _value = newValue;
    notifyListeners();
  }

  E fromJson(dynamic json);

  String toJson(E element) => jsonEncode(element);

  bool isEmpty() {
    return isValueEmpty() && errorMessage.isEmpty;
  }

  Future<void> fetch() async {
    try {
      _value = (await apiClient.fetch(this));
      errorMessage = "";
      alreadyFetch = true;
      notifyListeners();
    } on Exception catch (e) {
      errorMessage = e.toString();
      notifyListeners();
    }
  }

  Future<void> fetchNoNotify() async {
    try {
      _value = (await apiClient.fetch(this));
      errorMessage = "";
      alreadyFetch = true;
    } on Exception catch (e) {
      errorMessage = e.toString();
    }
  }

  void forceUpdate() {
    notifyListeners();
  }

}