
import 'package:bar_tender_api_dart/src/models/Model.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

abstract class EntityPage<E, M extends Model<E>> extends StatelessWidget {

  const EntityPage({super.key});

  Widget showEntity(BuildContext context, M model);
  Future<void> setModel(BuildContext context, M model) async {}

  Future<void> fetchData(BuildContext context, M model) async {
    final M model = Provider.of<M>(context);
    await setModel(context, model);
    if (model.isEmpty()) {
      await model.fetch();
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
        future: fetchData(context, Provider.of<M>(context)),
        builder: (context, snapshot) {
          return Center(
              child: Consumer<M>(
                builder: (context, model, child) {
                if (snapshot.connectionState == ConnectionState.waiting) {
                  return const CircularProgressIndicator();
                } else if (snapshot.hasError || model.errorMessage.isNotEmpty) {
                    return Column(
                        children:[
                          Text(model.errorMessage),
                          ElevatedButton(
                            onPressed: () {
                              model.fetch();
                              //remoteImageModel.fetchAsset();
                            },
                            child: const Text('reFetch'),
                          ),
                        ]
                    );
                  } else {
                    return RefreshIndicator(
                        onRefresh: () => model.fetch(),
                        child: showEntity(context, model)
                    );
                  }
                },
              )
          );
        }
    );
  }
}