import 'package:bar_tender_api_dart/src/models/Model.dart';
import 'package:bar_tender_api_dart/src/tenderUI/cardTender/CardTender.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

abstract class ListPage<E, M extends Model, C extends CardTender<E, M>> extends StatelessWidget {
  const ListPage({super.key});


  Widget intestazione() {
    return const Text("Seleziona l'elemento", style: TextStyle(fontWeight: FontWeight.bold, fontSize: 18));
  }

  C cardTenderBuilder(M model, E entity);
  Future<void> setModel(BuildContext context, M model) async {}

  Widget firstElement(BuildContext context, M model, E entity) {
    return Column(
      children: [
        intestazione(),
        cardTenderBuilder(model, entity),
      ],
    );
  }

  Future<void> fetchData(BuildContext context, M model) async {
    final M model = Provider.of<M>(context);
    await setModel(context, model);
    if (model.isEmpty()) {
      model.fetch();
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
        future: fetchData(context, Provider.of<M>(context)),
        builder: (context, snapshot) {
          return Consumer<M>(
            builder: (context, model, child) {
              if (snapshot.connectionState == ConnectionState.waiting) {
                return const CircularProgressIndicator();
              }
              else if (snapshot.hasError || model.errorMessage.isNotEmpty) {
                return Column(
                    children: [
                      Text(model.errorMessage),
                      ElevatedButton(
                        onPressed: () {
                          model.fetch();
                          //remoteImageModel.fetchAsset();
                        },
                        child: const Text('reFetch'),
                      ),
                    ]
                );
              } else {
                return displayEntity(model);
              }
            },
          );
        });
  }

  Widget displayEntity(M model) {
    return buildList(model, model.toList().cast());
  }

  Widget buildList(model, List<E> list) {
    if (list.isEmpty) {
      return const Column(
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          Text("Non ci sono elementi da selezionare!", style: TextStyle(fontSize: 20, fontWeight: FontWeight.bold),)
        ],
      );
    }
    return RefreshIndicator(
      onRefresh: () => model.fetch(),
      child: ListView.builder(
        itemCount: list.length,
        itemBuilder: (context, index) {
          final entity = list[index];
          var cardTender = cardTenderBuilder(model, entity);
          if (index == 0) {
            return firstElement(context, model, entity);
          }
          return cardTender;
        }
      )
    );
  }
}