import 'package:bar_tender_api_dart/src/models/MapModel.dart';
import 'package:bar_tender_api_dart/src/tenderUI/cardTender/CardTender.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

abstract class MapPage<E, K, M extends MapModel<K, E>, C extends CardTender<E, M>> extends StatelessWidget {
  const MapPage({super.key});

  C cardTenderBuilder(M model, E entity);

  @override
  Widget build(BuildContext context) {
    final M model = Provider.of<M>(context);
    if (model.value.isEmpty && model.errorMessage.isEmpty) {
      model.fetch();
    }

    return Center(
        child: Consumer<M>(
          builder: (context, model, child) {
            if (model.errorMessage.isNotEmpty){
              return Column(
                  children:[
                    Text(model.errorMessage),
                    ElevatedButton(
                      onPressed: () {
                        model.fetch();
                        //remoteImageModel.fetchAsset();
                      },
                      child: const Text('reFetch'),
                    ),
                  ]
              );
            } else if (model.value.isEmpty) {
              return const CircularProgressIndicator();
            } else {
              return RefreshIndicator(
                  onRefresh: () => model.fetch(),
                  child: ListView.builder(
                      itemCount: model.value.length,
                      itemBuilder: (context, index) {
                        final entity = model.value[index];
                        var cardTender = cardTenderBuilder(model, entity as E);
                        return cardTender;
                      }
                  )
              );
            }
          },
        )
    );
  }
}