import 'package:bar_tender_api_dart/src/customer/Customer.dart';
import 'package:bar_tender_api_dart/src/recipe_item/RecipeItem.dart';
import 'package:json_annotation/json_annotation.dart';

part 'RecipeItemCustom.g.dart';

@JsonSerializable()
class RecipeItemCustom extends RecipeItem {
  final int idCustomer;
  final String? codCustomer;

  RecipeItemCustom({
    required int id,
    required String? codRecipeItem,
    required int idDrink,
    required String? codDrink,
    required int idIngredient,
    required String? codIngredient,
    required this.idCustomer,
    required this.codCustomer,
    required double quantityPercent,
    required int sequencePosition,
  }) : super(
    id: id,
    codRecipeItem: codRecipeItem,
    idDrink: idDrink,
    codDrink: codDrink,
    idIngredient: idIngredient,
    codIngredient: codIngredient,
    quantityPercent: quantityPercent,
    sequencePosition: sequencePosition,
  );

  factory RecipeItemCustom.fromJson(Map<String, dynamic> json) =>
      _$RecipeItemCustomFromJson(json);

  @override
  Map<String, dynamic> toJson() => _$RecipeItemCustomToJson(this);

  factory RecipeItemCustom.empty() {
    return RecipeItemCustom(
      id: 0,
      codRecipeItem: '',
      idDrink: -1,
      codDrink: '',
      idIngredient: -1,
      codIngredient: '',
      idCustomer: -1,
      codCustomer: "",
      quantityPercent: 0.0,
      sequencePosition: 0,
    );
  }

  factory RecipeItemCustom.copy(RecipeItemCustom recipeItemCustom) {
    return RecipeItemCustom(
      id: recipeItemCustom.id,
      codRecipeItem: recipeItemCustom.codRecipeItem,
      idDrink: recipeItemCustom.idDrink,
      codDrink: recipeItemCustom.codDrink,
      idIngredient: recipeItemCustom.idIngredient,
      codIngredient: recipeItemCustom.codIngredient,
      idCustomer: recipeItemCustom.idCustomer,
      codCustomer: recipeItemCustom.codCustomer,
      quantityPercent: recipeItemCustom.quantityPercent,
      sequencePosition: recipeItemCustom.sequencePosition,
    );
  }

  factory RecipeItemCustom.fromRecipeToCustom(
      RecipeItem recipeItem, Customer customer) {
    return RecipeItemCustom(
      id: recipeItem.id,
      codRecipeItem: recipeItem.codRecipeItem,
      idDrink: recipeItem.idDrink,
      codDrink: recipeItem.codDrink,
      idIngredient: recipeItem.idIngredient,
      codIngredient: recipeItem.codIngredient,
      idCustomer: customer.id,
      codCustomer: customer.codCustomer,
      quantityPercent: recipeItem.quantityPercent,
      sequencePosition: recipeItem.sequencePosition,
    );
  }
}
