
import 'package:bar_tender_api_dart/src/models/Model.dart';
import 'package:flutter/material.dart';

abstract class CardTender<E, M extends Model> extends StatelessWidget {
  const CardTender({
    required this.model,
    required this.entity,
    this.onTap,
    this.startColor = Colors.blue,
    this.endColor = Colors.purple,
    this.startAlignment = Alignment.bottomCenter,
    this.endAlignment = Alignment.topCenter,
    Key? key,
  }) : super(key: key);

  final M model;
  final E entity;
  final Color startColor, endColor;
  final void Function(BuildContext context, E entity)? onTap;
  final Alignment startAlignment, endAlignment;

  onTapDefault (BuildContext context) {
    print('Implement onTap');
  }

  Widget displayEntity(BuildContext context);

  @override
  Widget build(BuildContext context) {
    return Card(
      clipBehavior: Clip.antiAliasWithSaveLayer,
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.circular(16.0),
      ),
      elevation: 4,
      margin: const EdgeInsets.all(10),
      child: Ink(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(16.0),
          gradient: LinearGradient(
            colors: [startColor, endColor],
            begin: startAlignment,
            end: endAlignment,
          ),
        ),
        child: InkWell(
          onTap: () {
            if (onTap == null) {
              onTapDefault(context);
            } else {
              onTap!(context, entity);
            }
          },
          child: Container(
            margin: const EdgeInsets.all(10),
            child: displayEntity(context),
          ),
        ),
      ),
    );
  }

}